package {
	import flash.display.Loader;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.ProgressEvent;
	import flash.geom.Rectangle;
	import flash.net.URLRequest;
	import flash.text.AntiAliasType;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	import flash.text.TextFormat;
	import flash.utils.getTimer;		

	/**
	 * @author Antti Kupila
	 */
	public class Image extends Sprite {
		
		// _________________________________________________________________________
		// VARIABLES
	
		private var imageLoader : Loader;
		private var path : String;
		
		private var label : TextField;
		
		private var size : Rectangle;
		private var parseTime : uint;
		
		private var loadStartTime : uint;

		// _________________________________________________________________________
		// CONSTRUCTOR
	
		public function Image( width : uint, height : uint, path : String, parseTime : uint ) {
			this.path = path;
			this.parseTime = parseTime;
			size = new Rectangle( 0, 0, width, height );
			
			graphics.beginFill( 0x333333 );
			graphics.drawRect( 0, 0, width, height);
			
			label = new TextField( );
			label.defaultTextFormat = new TextFormat( "_sans", 10, 0x333333, false );
			label.selectable = false;
			label.autoSize = TextFieldAutoSize.LEFT;
			label.antiAliasType = AntiAliasType.ADVANCED;
			label.y = height + 5;
			label.scaleX = label.scaleY = 1 / Main.IMAGE_SCALE;
			setText( "" );
			addChild( label );
			
			imageLoader = new Loader( );
			imageLoader.contentLoaderInfo.addEventListener( ProgressEvent.PROGRESS, loadProgressHandler );
			imageLoader.contentLoaderInfo.addEventListener( Event.COMPLETE, loadCompleteHandler );
			addChild( imageLoader );
		}
		
		// _________________________________________________________________________
		// PROTECTED METHODS
	
		private function setText( str : String ) : void {
			label.text = "Size: " + size.width + " x " + size.height + " parsed in " + parseTime + "ms  -  " + str;
		}
		
		// _________________________________________________________________________
		// EVENT HANDLERS
	
		protected function loadCompleteHandler( e : Event ) : void {
			imageLoader.alpha = 0;
			addEventListener( Event.ENTER_FRAME, enterFrameHandler );
			dispatchEvent( new Event( Event.COMPLETE ) );
			
			setText( "Image loaded in " + ( getTimer( ) - loadStartTime ) + "ms" );
		}
		
		protected function loadProgressHandler( e : ProgressEvent ) : void {
			setText( Math.round( e.bytesLoaded / e.bytesTotal * 100 ) + "%" );
		}
		
		protected function enterFrameHandler( e : Event ) : void {
			imageLoader.alpha += ( 1 - imageLoader.alpha ) * 0.2;
			if ( imageLoader.alpha > 0.95 ) {
				removeEventListener( Event.ENTER_FRAME, enterFrameHandler );
				imageLoader.alpha = 1;
			}
		}
		
		// _________________________________________________________________________
		// PUBLIC API METHODS
	
		public function load( ) : void {
			loadStartTime = getTimer( );
			imageLoader.load( new URLRequest( path ) );
		}
	}
}
