/*
Copyright (c) 2007 Antti Kupila.
URL: http://www.anttikupila.com

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
*/
package {
	import flash.display.LoaderInfo;
	import flash.display.Sprite;
	import flash.display.StageAlign;
	import flash.display.StageScaleMode;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.text.AntiAliasType;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	import flash.text.TextFormat;
	import flash.utils.getTimer;
	
	import com.anttikupila.utils.JPGSizeExtractor;	

	/**
	 * @author Antti Kupila
	 */
	public class Main extends Sprite {
		
		// _________________________________________________________________________
		// SETTINGS
	
		static public const PADDING_X : Number = 6;
		static public const PADDING_Y : Number = 5;
		static public const IMAGE_SCALE : Number = 0.25; // scale down to fit better..
		
		// _________________________________________________________________________
		// VARIABLES
	
		private var paths : Array;
		private var images  : Array;
		private var loadIndex : uint = 0;
		
		private var jpgSizeExtractor : JPGSizeExtractor;
		
		private var holder : Sprite;
		private var scroller : SimpleScrollbar;
		private var xPos : uint = 0;
		
		private var subtitle : TextField;
		
		private var loadStartTime : uint;	
		
		private var path : String = "";

		// _________________________________________________________________________
		// CONSTRUCTOR
	
		public function Main() {
			
			stage.align = StageAlign.TOP_LEFT;
			stage.scaleMode = StageScaleMode.NO_SCALE;
			
			paths = [ "image1.jpg", "image2.jpg", "image3.jpg", "image4.jpg", "image5.jpg", "image6.jpg", "image7.jpg", "image8.jpg", "image9.jpg", "image10.jpg" ];
			images = new Array( paths.length );
			
			jpgSizeExtractor = new JPGSizeExtractor( );
			jpgSizeExtractor.addEventListener( JPGSizeExtractor.PARSE_COMPLETE, sizeExtractHandler );
			
			var paramObj:Object = LoaderInfo( root.loaderInfo ).parameters;
			if ( paramObj.path != null ) path = paramObj.path;
			
			initUI( );
			
		}
		
		// _________________________________________________________________________
		// PRIVATE METHODS

		private function initUI( ) : void {
			var title : TextField = new TextField( );
			title.defaultTextFormat = new TextFormat( "_sans", 18, 0x020202, true );
			title.selectable = false;
			title.autoSize = TextFieldAutoSize.LEFT;
			title.text = "JPGSizeExtractor | Multiple image load example";
			title.x = PADDING_X;
			title.y = PADDING_Y;
			addChild( title );
			
			subtitle = new TextField( );
			subtitle.defaultTextFormat = new TextFormat( "_sans", 11, 0x333333, false );
			subtitle.selectable = false;
			subtitle.text = "Image meta data is parsed before loading the actual image, resulting in almost instantanious background boxes at the correct size, even if the image itself hasn't loaded.";
			subtitle.antiAliasType = AntiAliasType.ADVANCED;
			subtitle.wordWrap = true;
			subtitle.autoSize = TextFieldAutoSize.LEFT;
			subtitle.x = PADDING_X;
			subtitle.y = title.y + title.textHeight + 5;
			addChild( subtitle );
			
			holder = new Sprite( );
			addChild( holder );
			
			scroller = new SimpleScrollbar( holder, PADDING_X );
			scroller.visible = false; // show when first box shows
			addChild( scroller );
			
			stage.addEventListener( Event.RESIZE, resizeHandler );
			resizeHandler( null ); // update scrollbar position & subtitle size
			
			var loadButton : Sprite = new Sprite( );
			var loadButtonLabel : TextField = new TextField( );
			loadButtonLabel.defaultTextFormat = new TextFormat( "_sans", 14, 0x000000 );
			loadButtonLabel.selectable = false;
			loadButtonLabel.antiAliasType = AntiAliasType.ADVANCED;
			loadButtonLabel.autoSize = TextFieldAutoSize.LEFT;
			loadButtonLabel.text = "Click to load images!";
			loadButton.addChild( loadButtonLabel );
			loadButton.x = PADDING_X;
			loadButton.y = holder.y;
			loadButton.mouseChildren = false;
			addChild( loadButton );
			loadButton.addEventListener( MouseEvent.CLICK, loadStartHandler, false, 0, true );
		}

		private function loadNextImageMeta( ) : void {
			loadStartTime = getTimer( );
			jpgSizeExtractor.extractSize( path + paths[ loadIndex ], true );
		}
		
		private function loadNextFullsizeImage( ) : void {
			var image : Image = images[ loadIndex ];
			image.addEventListener( Event.COMPLETE, fullsizeLoadHandler );
			
			image.load( );
		}

		private function createImage( width : uint, height : uint ) : Image {
			var image : Image = new Image( width, height, path + paths[ loadIndex ], ( getTimer( ) - loadStartTime ) );
			image.x = xPos;
			image.scaleX = image.scaleY = IMAGE_SCALE;
			xPos += image.width + 20;
			holder.addChild( image );
			
			scroller.updateSize( );
			scroller.visible = true;
			
			return image;
		}
		
		private function loadFullsizeImages( ) : void {
			loadIndex = 0;
			loadNextFullsizeImage( );
		}

		// _________________________________________________________________________
		// EVENT HANDLERS
	
		private function sizeExtractHandler( e : Event ) : void {
			var target : JPGSizeExtractor = JPGSizeExtractor( e.target );
			trace( paths[ loadIndex ] + " size loaded at " + Math.round( target.loaded ) + " bytes" ); 
			images[ loadIndex ] = createImage( target.width, target.height );
			if ( ++loadIndex < paths.length ) {
				loadNextImageMeta( );
			} else {
				loadFullsizeImages( );
			}
		}
		
		private function resizeHandler( e : Event ) : void {
			scroller.y = stage.stageHeight - PADDING_Y - 6;
			subtitle.width = stage.stageWidth - PADDING_X * 2;
			holder.y = subtitle.y + subtitle.textHeight + 10;
		}
		
		private function fullsizeLoadHandler( e : Event ) : void {
			e.target.removeEventListener( Event.COMPLETE, fullsizeLoadHandler );
			if ( ++loadIndex < images.length ) {
				loadNextFullsizeImage( );			 
			} else {
				// All images loaded
				holder.cacheAsBitmap = true;
			}
		}
		
		private function loadStartHandler( e : MouseEvent ) : void {
			e.target.parent.removeChild( e.target );
			
			loadNextImageMeta( );
		}
	}
}