/*
Copyright (c) 2007 Antti Kupila.
URL: http://www.anttikupila.com

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
*/
package {
	import flash.display.DisplayObject;
	import flash.display.Shape;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.geom.Rectangle;		

	/**
	 * @author Antti Kupila
	 */
	public class SimpleScrollbar extends Sprite {
		
		// _________________________________________________________________________
		// VARIABLES
	
		private var target : DisplayObject;
		private var padding :  uint;
		private var bg : Shape;
		private var handle : Sprite;
		private var scrollWidth : Number;
		private var pos : Number = 0;
		private var currentPos : Number = 0;
		
		// _________________________________________________________________________
		// CONSTRUCTOR
	
		public function SimpleScrollbar( scrollTarget : DisplayObject, padding : uint ) {
			
			this.target = scrollTarget;
			this.padding = padding;
			
			scrollTarget.x = padding;
			
			bg = new Shape( );
			bg.graphics.beginFill( 0xFFFFFF, 0.2 );
			bg.graphics.drawRect( 0, -1, 100, 6 );
			bg.x = -padding;
			bg.blendMode = "invert";
			addChild( bg );
			
			handle = new Sprite( );
			handle.graphics.beginFill( 0xFF00FF, 0 );
			handle.graphics.drawRect( 0, -8, 10, 20 );
			handle.graphics.beginFill( 0x000000, 0.9 );
			handle.graphics.drawRect( 0, 0, 10, 4 );
			handle.addEventListener( MouseEvent.MOUSE_DOWN, mouseDownHandler );
			addChild( handle );
			
			x = padding;
			
			addEventListener( Event.ADDED_TO_STAGE, addedToStageHandler );
			addEventListener( Event.ENTER_FRAME, enterFrameHandler );
			
		}
		
		// _________________________________________________________________________
		// PROTECTED METHODS
	
		protected function update( ) : void {
			handle.width = Math.round( Math.min( scrollWidth / target.width, 1 ) * scrollWidth );
			
			handle.x = ( scrollWidth - handle.width ) * pos;
		}

		// _________________________________________________________________________
		// EVENT HANDLERS
	
		protected function addedToStageHandler( e : Event ) : void {
			removeEventListener( Event.ADDED_TO_STAGE, addedToStageHandler );
			stage.addEventListener( Event.RESIZE, resizeHandler );
		}
		
		protected function resizeHandler( e : Event ) : void {
			scrollWidth = stage.stageWidth - padding * 2;
			bg.width = stage.stageWidth;
			update( );
		}
		
		protected function mouseDownHandler( e : MouseEvent ) : void {
			stage.addEventListener( MouseEvent.MOUSE_UP, mouseUpHandler );
			stage.addEventListener( MouseEvent.MOUSE_MOVE, moveHandler );
			handle.startDrag( false, new Rectangle( 0, 0, scrollWidth - handle.width, 0 ) );
		}
		
		protected function mouseUpHandler( e : Event ) : void {
			stage.removeEventListener( MouseEvent.MOUSE_UP, mouseUpHandler );
			stage.removeEventListener( MouseEvent.MOUSE_MOVE, moveHandler );
			handle.stopDrag( );
		}
		
		protected function enterFrameHandler( e : Event ) : void {
			currentPos += ( pos - currentPos ) * 0.4;
			target.x = -currentPos * ( target.width - scrollWidth ) + padding;
		}
		
		protected function moveHandler( e : MouseEvent ) : void {
			pos = handle.x / ( scrollWidth - handle.width );
			if ( pos < 0 || isNaN( pos ) ) pos = 0;
			if ( pos > 1 ) pos = 1;
		}

		// _________________________________________________________________________
		// PUBLIC API METHODS
	
		public function updateSize( ) : void {
			try {
				resizeHandler( null );
			} catch ( e : Error ) {
				trace( e );
			}
		}
		
	}
}
